/* GNU polyxmass - the massist's program.
   -------------------------------------- 
   Copyright (C) 2000,2001,2002 Filippo Rusconi

   http://www.polyxmass.org

   This file is part of the "GNU polyxmass" project.
   
   The "GNU polyxmass" project is an official GNU project package (see
   www.gnu.org) released ---in its entirety--- under the GNU General
   Public License and was started at the Centre National de la
   Recherche Scientifique (FRANCE), that granted me the formal
   authorization to publish it under this Free Software License.

   This software is free software; you can redistribute it and/or
   modify it under the terms of the GNU  General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU  General Public
   License along with this software; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef PXMUTILS_NOTE_H
#define PXMUTILS_NOTE_H


#include "pxmutils-globals.h"
#include "pxmutils-prop.h"

typedef struct _PxmNote PxmNote;
typedef struct _PxmNoteVal PxmNoteVal;
typedef enum _PxmNoteValType PxmNoteValType;


enum
{
  NOTE_NAME,
  NOTE_VALUE,
  NOTE_BOTH
};

enum _PxmNoteValType
{
  PXM_NOTE_VAL_TYPE_STR = 0,
  PXM_NOTE_VAL_TYPE_INT = 1,
  PXM_NOTE_VAL_TYPE_DBL = 2
};


/* A PxmNote object is made of one compulsory 'name' string and of
   an array of PxmNoteVal objects. While the array must exist, it can
   be empty.
*/
struct _PxmNote
{
  gchar *name;
  
  GPtrArray *notevalGPA;
};

struct _PxmNoteVal
{
  PxmNoteValType type;
  gpointer *value;
};


/* HELPER FUNCTIONS
 */



/* NEW'ING FUNCTIONS, DUPLICATING FUNCTIONS ...
 */
PxmNote*
pxmutils_note_new (void);

PxmNoteVal*
pxmutils_noteval_new (void);

PxmProp*
pxmutils_note_prop_new (void);

PxmNote *
pxmutils_note_dup (PxmNote *note);

PxmProp*
pxmutils_note_prop_dup (PxmProp *prop);



/*  LOCATING FUNCTIONS
 */
gint
pxmutils_noteval_get_index_by_noteval (PxmNoteVal *noteval, GPtrArray *GPA);



/* DATA MODIFYING FUNCTIONS
 */
gchar *
pxmutils_note_set_name (PxmNote *note, gchar *name);



/* COMPARISON FUNCTIONS
 */
gint
pxmutils_note_cmp (gint what /*NAME|VALUE|BOTH*/, 
		   PxmNote *note1, PxmNote *note2);

gint
pxmutils_noteval_cmp (PxmNoteVal *noteval1,
		      PxmNoteVal *noteval2);

gint
pxmutils_note_prop_cmp (gint what /*NAME|VALUE|BOTH*/,
			PxmProp *prop1,
			PxmProp *prop2);

gint
pxmutils_note_prop_data_cmp (gpointer data1,
			     gpointer data2);




/* RENDERING FUNCTIONS - FORMATTER FUNCTIONS
 */
gchar *
pxmutils_note_prop_format_xml_node_note (PxmProp *prop,
					 gchar *indent,
					 gint offset,
					 gpointer user_data);

PxmProp *
pxmutils_note_prop_render_xml_node_note (xmlDocPtr xml_doc,
					 xmlNodePtr xml_node,
					 gpointer user_data);



/* FREE'ING FUNCTIONS
 */
gint 
pxmutils_note_free (PxmNote *note);

gint 
pxmutils_noteval_GPA_empty (GPtrArray *GPA);

gint 
pxmutils_noteval_free (PxmNoteVal *noteval);

gint 
pxmutils_note_prop_free (PxmProp *prop);

gint 
pxmutils_note_prop_free_but_not_data (PxmProp *prop);












#endif
/* ifndef PXMUTILS_NOTE_H
 */





