/* GNU polyxmass - the massist's program.
   -------------------------------------- 
   Copyright (C) 2000,2001,2002 Filippo Rusconi

   http://www.polyxmass.org

   This file is part of the "GNU polyxmass" project.
   
   The "GNU polyxmass" project is an official GNU project package (see
   www.gnu.org) released ---in its entirety--- under the GNU General
   Public License and was started at the Centre National de la
   Recherche Scientifique (FRANCE), that granted me the formal
   authorization to publish it under this Free Software License.

   This software is free software; you can redistribute it and/or
   modify it under the terms of the GNU  General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU  General Public
   License along with this software; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef PXMUTILS_XXXXSPEC_H
#define PXMUTILS_XXXXSPEC_H


#include "pxmutils-globals.h"



typedef struct _PxmPolymerspec PxmPolymerspec;

typedef struct _PxmMonomerspec PxmMonomerspec;

enum
  {
    ERR_MNMSPEC_NOERR,
    ERR_MNMSPEC_NO_CODE,
    ERR_MNMSPEC_CODE_BUT_NO_MODIF,
    ERR_MNMSPEC_NO_VECTOR_FILE,
    ERR_MNMSPEC_NO_RASTER_FILE,
    ERR_MNMSPEC_NO_VECTOR_NOR_RASTER_FILES
  };


enum
  {
    FILE_VECTOR,
    FILE_RASTER,
    FILE_NONE,
    FILE_ANY,
    FILE_BOTH
  };



struct _PxmPolymerspec
{
  /* Name of the polymer definition (ie polymer type, 'protein').
   */
  gchar *type;

  /* Polymer definition file (this is a relative name, like
     'protein.xml').
   */
  gchar *file;

  /* Polymer definition directory (mainly where pixmaps and accessory
   * files are stored; this is a relative name, like 'protein').
   */
  gchar *dir;
};
  

struct _PxmMonomerspec
{
  /* Name of the raster graphic file (png or jpg, for example).
   */
  gchar *raster;

  /* Name of the vector graphic file (svg).
   */
  gchar *vector;

  /* Code of the monomer to which the graphic file is associated.
   */
  gchar *code;

  /* Name of the modif to which the graphic file is associated.
   */
  gchar *modif;

  /* Graphical action to be performed using the graphic file (upon
   * modification of a monomer).
   */
  gchar *action;
};



/* NEW'ING FUNCTIONS, DUPLICATING FUNCTIONS ...
 */
PxmPolymerspec *
pxmutils_polymerspec_new (void);

gboolean
pxmutils_polymerspec_set_type (PxmPolymerspec *ps, gchar *type);

gboolean
pxmutils_polymerspec_set_file (PxmPolymerspec *ps, gchar *file);

gboolean
pxmutils_polymerspec_set_dir (PxmPolymerspec *ps, gchar *dir);




PxmMonomerspec *
pxmutils_monomerspec_new (void);

gboolean
pxmutils_monomerspec_set_raster (PxmMonomerspec *ms, gchar *file);

gboolean
pxmutils_monomerspec_set_vector (PxmMonomerspec *ms, gchar *file);

gboolean
pxmutils_monomerspec_set_code (PxmMonomerspec *ms, gchar *code);

gboolean
pxmutils_monomerspec_set_modif (PxmMonomerspec *ms, gchar *modif);

gboolean
pxmutils_monomerspec_set_action (PxmMonomerspec *ms, gchar *action);



/*  LOCATING FUNCTIONS
 */
gint
pxmutils_polymerspec_get_index_by_type (GPtrArray *GPA, gchar *type);

gint
pxmutils_polymerspec_get_index_top_by_type (GPtrArray *GPA, gchar *type);

gint
pxmutils_polymerspec_get_index_bottom_by_type (GPtrArray *GPA, gchar *type);

PxmPolymerspec *
pxmutils_polymerspec_get_ptr_by_type (GPtrArray *GPA, gchar *type);

PxmPolymerspec *
pxmutils_polymerspec_get_ptr_top_by_type (GPtrArray *GPA, gchar *type);

PxmPolymerspec *
pxmutils_polymerspec_get_ptr_bottom_by_type (GPtrArray *GPA, gchar *type);






PxmMonomerspec *
pxmutils_monomerspec_get_ptr_by_code_modif (gchar *code, gchar* modif,
					    gint check, 
					    GPtrArray *GPA,
					    gint *error);
gint
pxmutils_monomerspec_get_index_by_code (GPtrArray *GPA, gchar *code);

gint
pxmutils_monomerspec_get_index_top_by_code (GPtrArray *GPA, gchar *code);

gint
pxmutils_monomerspec_get_index_bottom_by_code (GPtrArray *GPA, gchar *code);

gint
pxmutils_monomerspec_get_index_by_modif (GPtrArray *GPA, gchar *modif);

gint
pxmutils_monomerspec_get_index_top_by_modif (GPtrArray *GPA, gchar *modif);

gint
pxmutils_monomerspec_get_index_bottom_by_modif (GPtrArray *GPA, 
						gchar *modif);


/* UTILITY FUNCTIONS
 */
gboolean
pxmutils_polymerspec_reset (PxmPolymerspec *ps);

gint
pxmutils_polymerspec_fill_array_from_dict (gchar *file,
					   GPtrArray *GPA);

gboolean
pxmutils_polymerspec_init_from_line (gchar *line,
				     gchar *dir,
				     PxmPolymerspec *ps) ;

gboolean
pxmutils_polymerspec_init_from_dict_with_type (gchar *type,
					       gchar *file,
					       PxmPolymerspec *ps);






gint
pxmutils_monomerspec_fill_array_from_dict (gchar *file,
					   GPtrArray *GPA);

PxmMonomerspec *
pxmutils_monomerspec_parse_mnm_line (gchar *line, gchar *dir);


PxmMonomerspec *
pxmutils_monomerspec_parse_mdf_line (gchar *line, gchar *dir);




/* FREE'ING FUNCTIONS
 */
gint
pxmutils_polymerspec_free (PxmPolymerspec *ps);



gint
pxmutils_monomerspec_free (PxmMonomerspec *ms);


/* GPtrArray-RELATED FUNCTIONS
 */
gint
pxmutils_polymerspec_GPA_free (GPtrArray *GPA);



gint
pxmutils_monomerspec_GPA_empty (GPtrArray *GPA);

gint
pxmutils_monomerspec_GPA_free (GPtrArray *GPA);
















#endif





